"""Top-level package for pyutils.

A comprehensive Python utility library providing functions for array manipulation,
string processing, mathematical operations, object handling, function utilities,
asynchronous operations, and byte processing.

Ported from the jsutils JavaScript library to provide similar functionality in Python.
"""

__author__ = """mudssky"""
__email__ = "mudssky@gmail.com"
__version__ = "0.2.0"

# Import all modules
from . import array, async_utils, collection, function, math, object, string
from . import bytes as bytes_utils

# Import commonly used functions for convenience
from .array import (
    alphabetical,
    chunk,
    diff,
    first,
    has_intersects,
    last,
    range_iter,
    range_list,
    shuffle,
    toggle,
    unique,
    zip_object,
)
from .async_utils import (
    delay,
    filter_async,
    map_async,
    race,
    retry_async,
    run_in_thread,
    sleep_async,
    timeout,
)
from .bytes import (
    Bytes,
    bytes_util,
    humanize_bytes,
    parse_bytes,
)
from .collection import (
    at,
    copy_within,
    entries,
    every,
    fill,
    find_index,
    find_last_index,
    flat_map,
    group_by,
    includes,
    keys,
    some,
    splice,
    to_reversed,
    to_sorted,
    values,
    with_item,
)
from .function import (
    debounce,
    memoize,
    once,
    throttle,
    with_retry,
)
from .math import (
    clamp,
    factorial,
    fibonacci,
    gcd,
    get_random_int,
    is_even,
    is_odd,
    is_prime,
    lcm,
    lerp,
    normalize,
)
from .object import (
    deep_copy,
    flatten_dict,
    get_nested_value,
    merge,
    omit,
    pick,
    safe_json_stringify,
    set_nested_value,
)
from .string import (
    camel_case,
    capitalize,
    dash_case,
    fuzzy_match,
    generate_uuid,
    parse_template,
    pascal_case,
    slugify,
    snake_case,
    trim,
    truncate,
)


# Define what gets exported when using "from pyutils import *"
__all__ = [
    "Bytes",
    "alphabetical",
    "array",
    "async_utils",
    "at",
    "bytes_util",
    "bytes_utils",
    "camel_case",
    "capitalize",
    "chunk",
    "clamp",
    "collection",
    "copy_within",
    "dash_case",
    "debounce",
    "deep_copy",
    "delay",
    "diff",
    "entries",
    "every",
    "factorial",
    "fibonacci",
    "fill",
    "filter_async",
    "find_index",
    "find_last_index",
    "first",
    "flat_map",
    "flatten_dict",
    "function",
    "fuzzy_match",
    "gcd",
    "generate_uuid",
    "get_nested_value",
    "get_random_int",
    "group_by",
    "has_intersects",
    "humanize_bytes",
    "includes",
    "is_even",
    "is_odd",
    "is_prime",
    "keys",
    "last",
    "lcm",
    "lerp",
    "map_async",
    "math",
    "memoize",
    "merge",
    "normalize",
    "object",
    "omit",
    "once",
    "parse_bytes",
    "parse_template",
    "pascal_case",
    "pick",
    "race",
    "range_iter",
    "range_list",
    "retry_async",
    "run_in_thread",
    "safe_json_stringify",
    "set_nested_value",
    "shuffle",
    "sleep_async",
    "slugify",
    "snake_case",
    "some",
    "splice",
    "string",
    "throttle",
    "timeout",
    "to_reversed",
    "to_sorted",
    "toggle",
    "trim",
    "truncate",
    "unique",
    "values",
    "with_item",
    "with_retry",
    "zip_object",
]
